;________________________________________________________________________
;
;			  B E A C O N . A S M
;________________________________________________________________________
;
; APRS beacon for Pic-Encoder - By IK3SVW Max - Version 1.0 - July 2002 -
; Based on some Byon Garrabrant N6BG routines.
;________________________________________________________________________

	LIST	P=PIC16F84
	include	"P16F84.INC"
	__CONFIG _HS_OSC & _CP_OFF & _PWRTE_ON & _WDT_OFF

;________________________________________________________________________

	cblock	H'0C'			; system variables
		beacon_counter
		status_counter
		hex1
		hex2
		temp
		temp2
	endc
;________________________________________________________________________

#define	PTT_IN		PORTA, 3
#define	PTT_OUT		PORTA, 2
#define	VALID_OUT	PORTB, 0
#define	VOX_IN		PORTB, 3
#define	OPTION_IN	PORTB, 4
;________________________________________________________________________

	ORG	0			; reset vector
	goto	main
;________________________________________________________________________

	include	"OPTIONS.ASM"		; User options
	ORG	0x04
	include	"MX614.ASM"		; MX614 Modem code
	include "AX25.ASM"		; AX-25 routines
	include "TIMING.ASM"		; Timings routines
;________________________________________________________________________

sendHeader
	movlw	OPT_CALLSIGN_DIGI
	movwf	temp
Packet_Send_Callsign
	movfw	temp
	call	Get_Option
	iorlw	0
	skpnz
	 goto	Packet_Send_Type
	movwf	temp2
	incf	temp, W
	call	Get_Option
	iorlw	0
	clrc
	skpnz
	 setc
 	rlf	temp2, W
	call	AX25_Send_Data
	incf	temp, F
	goto	Packet_Send_Callsign
Packet_Send_Type
	movlw	H'03'
	call	AX25_Send_Data
	movlw	H'F0'
	goto	AX25_Send_Data
;________________________________________________________________________

Packet_Send_String
	movfw	hex1
	call	Get_Option
	iorlw	0
	skpnz
	 return
	call	AX25_Send_Data
	incf	hex1, F
	goto	Packet_Send_String
;________________________________________________________________________

SendStatus				; send status string
	call	AX25_Send_Start
	call	sendHeader
	movlw	'>'
	call 	AX25_Send_Data
	movlw	OPT_STATUS_OFFSET
	call	Get_Option
	movwf	hex1
	call	Packet_Send_String
	goto	AX25_Send_End
;________________________________________________________________________

SendBeacon				; send beacon string
	call	AX25_Send_Start

SendBeaconManual
	call	sendHeader
	movlw	OPT_BEACON_OFFSET
	call	Get_Option
	movwf	hex1
	call	Packet_Send_String
	goto	AX25_Send_End
;________________________________________________________________________

BigDelay				; long delay routine for
	clrf	hex1			; flashing LED at startup
bigDly1	clrf	hex2
bigDly2	decfsz	hex2, f
	 goto bigDly2
	decfsz	hex1, f
	 goto	bigDly1
	return
;________________________________________________________________________
	
main	
	call	Tone_Init		; Initialization
	call	Timing_Init

	bsf	STATUS, RP0		; Select Bank 1
	bcf	PTT_OUT			; make PTT an output
	bcf	VALID_OUT		; make Valid Led an output
	bcf	STATUS, RP0		; Select Bank 0
	bcf	PTT_OUT			; release PTT


	movlw	D'5'
	movwf	temp
flash	bsf	VALID_OUT		; flash valid led on reset
	call	BigDelay
	bcf	VALID_OUT
	call	BigDelay
	decfsz	temp, f
	 goto	flash
;________________________________________________________________________

mainLoop
	call	Timing_Check		; check timing and inputs	
	iorlw	0			; look at result
	skpnz				; if zero
	 goto	mainLoop		;  continue loop

	bsf	VALID_OUT		; set valid led on
	decfsz	beacon_counter, f	; beacon timing
	 goto	bcntest
	movlw	OPT_BEACON_RATE1
	btfss	OPTION_IN
	 movlw	OPT_BEACON_RATE2
	call	Get_Option
	movwf	beacon_counter		; reset beacon counter
	bsf	PTT_OUT			; press PTT
	call	SendBeacon		; send the beacon packet

bcntest	decfsz	status_counter, f	; satus timing
	 goto	nobcn
	movlw	OPT_STATUS_RATE1
	btfss	OPTION_IN
	 movlw	OPT_STATUS_RATE2
	call	Get_Option
	movwf	status_counter		; reset status counter
	bsf	PTT_OUT			; press PTT
	call	SendStatus		; send the status packet

nobcn	bcf	PTT_OUT			; release PTT
	call	BigDelay		; wait a while with valid led on
	bcf	VALID_OUT		; and set valid led off
	goto	mainLoop		; continue loop
;________________________________________________________________________
	
	END
;________________________________________________________________________